﻿/*
 * PCD8544.c
 * Модуль для работы с дисплеями на базе контроллера PCD8544.
 *
 * Author: Погребняк Дмитрий
 * 
 * Помещённый здесь код является свободным. Т.е. допускается его свободное использование для любых целей, включая коммерческие, при условии указания ссылки на автора (Погребняк Дмитрий, http://aterlux.ru/).
 * Code included here is free to use. It can be used for free for any purpose, including commercial use, as long as link to author (Pogrebnyak Dmitry, http://aterlux.ru/) is included.
 */ 

#include <avr/io.h>
#include "PCD8544.h"
#include "tools.h"
#include <util/delay.h>
#include <avr/pgmspace.h>

#ifdef PCD8544_SOFTWARE
  void inline pcd8544_sw_send(uint8_t data) {
    for (uint8_t i = 0; i < 8; i++) {
      PCD8544_SW_SCK_PORT &= ~PCD8544_SW_SCK;
      if (data & 0x80) {
        PCD8544_SW_MOSI_PORT |= PCD8544_SW_MOSI;
      } else {
        PCD8544_SW_MOSI_PORT &= ~PCD8544_SW_MOSI;
      }
      data <<= 1;
      PCD8544_SW_SCK_PORT |= PCD8544_SW_SCK;
    }
  }
#endif

inline void pcd8544_reset_assert() {
  PCD8544_RESET_PORT &= ~PCD8544_RESET;
}

inline void pcd8544_reset_deassert() {
  PCD8544_RESET_PORT |= PCD8544_RESET;
}

inline void pcd8544_sce_assert() {
  PCD8544_SCE_PORT &= ~PCD8544_SCE;
}

inline void pcd8544_sce_deassert() {
  PCD8544_WAIT_DONE;
  PCD8544_SCE_PORT |= PCD8544_SCE;
}

void display_data(uint8_t data) {
  PCD8544_WAIT_DONE;
  PCD8544_DC_PORT |= PCD8544_DC;
  PCD8544_SEND(data);
}

void display_cmd(uint8_t data) {
  PCD8544_WAIT_DONE;
  pcd8544_sce_deassert(); // Чтобы не терялась синхронизация, периодически будем сбрасывать линию sce 
  PCD8544_DC_PORT &= ~PCD8544_DC;
  pcd8544_sce_assert();
  PCD8544_SEND(data);
}

// Инициализирует дисплей
void display_init() {
#ifdef PCD8544_SOFTWARE
  PCD8544_SW_SCK_PORT |= PCD8544_SW_SCK;
#endif 
  pcd8544_reset_assert();
  _delay_us(10);
  pcd8544_reset_deassert();
  _delay_us(10);
  
  pcd8544_sce_assert();
  display_cmd(0x21);	// set LCD mode
  display_cmd(0x80 + 58);	// set bias voltage
  
  display_cmd(0x04);	// temperature correction
  display_cmd(0x13);	// 1:48
  display_cmd(0x20);	
  display_cmd(0x0c);	// set LCD mode,display normally
}

// Готовит контроллер дисплея и затем переводит его в режим экономии энергии
void display_powerdown() {
  display_clear();
  display_cmd(0x21);
  display_cmd(0x80); // отключение генератора напряжения
  display_cmd(0x24);
  display_cmd(0x08);
  PCD8544_WAIT_DONE;
  PCD8544_SW_MOSI_PORT |= PCD8544_SW_MOSI;
  pcd8544_sce_deassert();
}

// Выбирает страницу (позицию по вертикали) для вывода
void display_setpage(uint8_t page) {
  display_cmd(0x40 | (page & 7));
}

// Выбирает горизонтальную позицию для вывода
void display_setx(uint8_t x) {
  display_cmd(0x80 | x);
}

// Выбирает страницу и горизонтальную позицию для вывода
void display_setpos(uint8_t page, uint8_t x) {
  display_cmd(0x40 | (page & 7));
  display_cmd(0x80 | x);
}

/* Заполняет экран заданными чередующимися значениями */
void display_fill(uint8_t oddData, uint8_t evenData) {
  display_setpos(0, 0);
  for (uint8_t y = 0; y < DISPLAY_PAGES; y++) {
    for (uint8_t x = 0; x < DISPLAY_WIDTH; x += 2) {
      display_data(oddData);
      display_data(evenData);
    }    
  }
}

/* Очищает экран, устанавливает курсор в левый верхний угол */
void display_clear() {
  display_setpos(0, 0);
  for (uint8_t y = 0; y < DISPLAY_PAGES; y++) {
    for (uint8_t x = 0; x < DISPLAY_WIDTH; x++) {
      display_data(0);
    }    
  }
  display_setpos(0, 0);
}

/* Выводит на экран строку из флеш-памяти
 * page - строка на которую выводится изображение
 * x - позиция, начиная с которой выводится изображение
 * pData - ссылка на данные во флеш-памяти
 * width - количество выводимых данных, ширина
 * xorMask - инверсия, применяемая к выводимым данным.
 * yinvert - если не ноль, переворачивает изображение вверх-ногами
 * 
 * Если изображение не помещается на экран по ширине, выводит только его часть. Возвращает сколько колонок было выведено
 */
uint8_t display_sprite(uint8_t page, uint8_t x, PGM_VOID_P pData, uint8_t width, uint8_t xorMask, uint8_t yinvert) {
  if ((page >= DISPLAY_PAGES) || (x >= DISPLAY_WIDTH))
    return 0;
  display_setpos(page, x);
  uint8_t r = DISPLAY_WIDTH - x;
  if (width > r)
    width = r;
  else 
    r = width;
  if (yinvert) {
    while (width--) {
      display_data(reversebits(pgm_read_byte(pData++)) ^ xorMask);
    }  
  } else {  
    while (width--) {
      display_data(pgm_read_byte(pData++) ^ xorMask);
    }  
  }  
  return r;
}

/* Копирует из буфера изображение на экран
 * page, x - начальная позиция для вывода буфера
 * pData - указатель на буфер
 * pages_height - высота буфера (в страницах)
 * width - ширина буфера
 */
void display_buffer(uint8_t page, uint8_t x, uint8_t * pData, uint8_t pages_height, uint8_t width) {
  if ((x >= DISPLAY_WIDTH) || (page >= DISPLAY_PAGES))
    return;
  uint8_t w = DISPLAY_WIDTH - x;
  if (w > width)
    w = width;
  uint8_t t = page + ((pages_height > DISPLAY_PAGES) ? DISPLAY_PAGES : pages_height);
  if (t > DISPLAY_PAGES)
    t = DISPLAY_PAGES;
  while (page < t) {
    display_setpos(page++, x);
    for (uint8_t i = 0; i < w; i++) {
      display_data(pData[i]);
    }
    pData += width;
  }
}