﻿/*
 * PCD8544.h
 *
 * Модуль для работы с дисплеями на базе контроллера PCD8544.
 * Module for work with displays based on PCD8544 controller
 *
 * Author: Погребняк Дмитрий
 * 
 * Помещённый здесь код является свободным. Т.е. допускается его свободное использование для любых целей, включая коммерческие, при условии указания ссылки на автора (Погребняк Дмитрий, http://aterlux.ru/).
 * Code included here are free to use. It can be used for free for any purpose, including commercial use, as long as link to author (Pogrebnyak Dmitry, http://aterlux.ru/) is included.
 */ 


#ifndef PCD8544_H_
#define PCD8544_H_

#define F_CPU 8000000UL
#include <avr/io.h>
#include <avr/pgmspace.h>

// Порт и номер пина, на который выводится !RESET сигнал
// Port and Pin number for !RESET signal
#define PCD8544_RESET_PORT PORTD
#define PCD8544_RESET_PIN_NUM 4
#define PCD8544_RESET (1 << PCD8544_RESET_PIN_NUM)

// Порт и номер пина, на который выводится !SCE (CS) сигнал
// Port and Pin number for !SCE (CS) signal
#define PCD8544_SCE_PORT PORTD
#define PCD8544_SCE_PIN_NUM 3
#define PCD8544_SCE (1 << PCD8544_SCE_PIN_NUM)

// Порт и номер пина, на который выводится D/!C сигнал
// Port and Pin number for D/!C signal
#define PCD8544_DC_PORT PORTD
#define PCD8544_DC_PIN_NUM 2
#define PCD8544_DC (1 << PCD8544_DC_PIN_NUM)

// Определить для программного формирования 
// If defined, then software output
#define PCD8544_SOFTWARE

#ifdef PCD8544_SOFTWARE
 // Порт и номер пина, на который выводится сигнал MOSI (DIN на дисплее) 
 // Port and Pin number for software MOSI (DIN on display controller) signal
 #define PCD8544_SW_MOSI_PORT PORTD
 #define PCD8544_SW_MOSI_PIN_NUM 1
 #define PCD8544_SW_MOSI (1 << PCD8544_SW_MOSI_PIN_NUM)
 
 // Порт и номер пина, на который выводится сигнал SCK
 // Port and Pin number for software SCK signal
 #define PCD8544_SW_SCK_PORT PORTD
 #define PCD8544_SW_SCK_PIN_NUM 0
 #define PCD8544_SW_SCK (1 << PCD8544_SW_SCK_PIN_NUM)

 // Пустая функция ожидания 
 // Void wait function
 #define PCD8544_WAIT_DONE
 
 // Функция для передачи одного байта
 // A byte transmission function
 #define PCD8544_SEND(x) pcd8544_sw_send(x)
 
#else 
// Если не задана программная передача, то здесь описываются методы обращения к регистрам периферии
// Периферия должна быть настроена для передачи сначала MSB, 3 режим SPI (POL=1 PHA=1).
// Скорость обмена не должна превышать 4Мгц
// If software transmission not defined, here peripheral registers should be described.
// Peripheral should be configured to transmit MSB first, SPI-3 mode (POL=1, PHA=1)
// Transmission rate should not exceed 4MHz
// 
// Для использования USART в режиме SPI MASTER
// Определение процедуры передачи байта
// Define a byte transmission procedure
 #define PCD8544_SEND(x) UDR = x
 
 // Процедура ожидания окончания асинхронной передачи
 // Procedure for awaiting of asynchronous transmission to complete
 #define PCD8544_WAIT_DONE while (!(UCSRA & (1 << TXC));

#endif

#define DISPLAY_WIDTH 84
#define DISPLAY_PAGES 6
#define DISPLAY_HEIGHT (DISPLAY_PAGES * 8)

/* Инициализирует дисплей
 * Все задействованные порты должны быть настроены заранее.
 * Если используется аппаратная передача, то соответствующая периферия должна быть подготовлена до вызова
 * display_init
 * 
 * Initializes the display.
 * All used ports should be configured before calling of this function.
 * Also, if hardware transmission engaged, all corresponding peripherials should be configured.
 */
void display_init();

/* Готовит контроллер дисплея и затем переводит его в режим экономии энергии.
 * Устанавливает на всех портах дисплея высокий уровень.
 * Для повторной инициализации дисплея необходимо вызвать  display_init() вновь
 * 
 * Preapares the display controller, and put it to power-down mode.
 * All used ports is set up to high level.
 * To resume from power-down, call display_init() again
 */
void display_powerdown();



/* Передаёт графические данные на дисплей 
 * Transmits a graphical data byte */
void display_data(uint8_t t);

/* Передаёт команду дисплею 
 * Transmits a command to display controller */
void display_cmd(uint8_t t);

/* Выбирает страницу (позицию по вертикали) для вывода 
 * page - страница от 0 до 5
 * 
 * Selects page (vertical postion) from 0 to 5 */
void display_setpage(uint8_t page);

/* Выбирает горизонтальную позицию для вывода
 * x - позиция от 0 до 83 
 * 
 * Selects horizontal position from 0 to 83 */
void display_setx(uint8_t x);

/* Выбирает страницу и горизонтальную позицию для вывода
 * page - страница от 0 до 5
 * x - позиция от 0 до 83 
 * 
 * Selects both page and horizontal position at once
 * */
void display_setpos(uint8_t page, uint8_t x);

/* Заполняет экран заданными чередующимися значениями 
 * Fills the screen with interleaved values */
void display_fill(uint8_t oddData, uint8_t evenData);

/* Очищает экран, устанавливает курсор в левый верхний угол 
 * Clears the screen. Moves cursor to the left upper corner */
void display_clear();

/* Выводит на экран строку из флеш-памяти
 * page - строка на которую выводится изображение
 * x - позиция, начиная с которой выводится изображение
 * pData - ссылка на данные во флеш-памяти
 * width - количество выводимых данных, ширина
 * xorMask - инверсия, применяемая к выводимым данным.
 * yinvert - если не ноль, переворачивает изображение вверх-ногами
 * 
 * Если изображение не помещается на экран по ширине, выводит только его часть. Возвращает сколько колонок было выведено
 */
uint8_t display_sprite(uint8_t page, uint8_t x, PGM_VOID_P pData, uint8_t width, uint8_t xorMask, uint8_t yinvert);

/* Копирует из буфера изображение на экран
 * page, x - начальная позиция для вывода буфера
 * pData - указатель на буфер
 * pages_height - высота буфера (в страницах)
 * width - ширина буфера
 * 
 * Transmits a data from the offscreen buffer to display region.
 * page, x - initial position for buffer output
 * pData - pointer to the buffer
 * pages_height - height of the buffer in pages
 * width - width of the buffer area
 */
void display_buffer(uint8_t page, uint8_t x, uint8_t * pData, uint8_t pages_height, uint8_t width);

#endif /* PCD8544_H_ */